@@ -15,10 +15,11 @@ You should have received a copy of the GNU Affero General Public License
 along with Silkaj. If not, see <https://www.gnu.org/licenses/>.
 """
 
-SILKAJ_VERSION = "0.6.5"
-NO_MATCHING_ID = "No matching identity"
+SILKAJ_VERSION = "0.7.6"
 G1_SYMBOL = "Ğ1"
 GTEST_SYMBOL = "ĞTest"
 G1_DEFAULT_ENDPOINT = "g1.duniter.org", "443"
 G1_TEST_DEFAULT_ENDPOINT = "g1-test.duniter.org", "443"
 CONNECTION_TIMEOUT = 10
+ASYNC_SLEEP = 0.1
+SOURCES_PER_TX = 40
