@@ -16,23 +16,29 @@ along with Silkaj. If not, see <https://www.gnu.org/licenses/>.
 """
 
 import webbrowser
-from pydoc import pager
-from sys import exit
+from click import command, echo_via_pager, confirm
 
 
 def license_approval(currency):
     if currency != "g1":
         return
+    if confirm(
+        "You will be asked to approve Ğ1 license. Would you like to display it?"
+    ):
+        display_license()
+    confirm("Do you approve Ğ1 license?", abort=True)
+
+
+@command("license", help="Display Ğ1 license")
+def license_command():
     display_license()
-    if input("Do you approve Ğ1 license [yes/no]? ") != "yes":
-        exit(1)
 
 
 def display_license():
     language = input("In which language would you like to display Ğ1 license [en/fr]? ")
     if language == "en":
         if not webbrowser.open("https://duniter.org/en/wiki/g1-license/"):
-            pager(open("licence-G1/license/license_g1-en.rst").read())
+            echo_via_pager(open("licence-G1/license/license_g1-en.rst").read())
     else:
         if not webbrowser.open("https://duniter.org/fr/wiki/licence-g1/"):
-            pager(open("licence-G1/license/license_g1-fr-FR.rst").read())
+            echo_via_pager(open("licence-G1/license/license_g1-fr-FR.rst").read())
