@@ -17,53 +17,72 @@ along with Silkaj. If not, see <https://www.gnu.org/licenses/>.
 
 from __future__ import unicode_literals
 from ipaddress import ip_address
-from json import loads
 import socket
-import urllib.request
 import logging
 from sys import exit, stderr
-import click
+from click import pass_context
+from asyncio import sleep
+from duniterpy.api.client import Client
+from duniterpy.api.bma import blockchain, network
 
 from silkaj.constants import (
     G1_DEFAULT_ENDPOINT,
     G1_TEST_DEFAULT_ENDPOINT,
     CONNECTION_TIMEOUT,
+    ASYNC_SLEEP,
 )
 
 
-def discover_peers(discover):
+async def discover_peers(discover):
     """
     From first node, discover his known nodes.
     Remove from know nodes if nodes are down.
     If discover option: scan all network to know all nodes.
         display percentage discovering.
     """
-    endpoints = parse_endpoints(get_request("network/peers")["peers"])
+    client = ClientInstance().client
+    endpoints = await get_peers_among_leaves(client)
     if discover:
         print("Discovering network")
     for i, endpoint in enumerate(endpoints):
         if discover:
             print("{0:.0f}%".format(i / len(endpoints) * 100))
-        if best_node(endpoint, False) is None:
+        if best_endpoint_address(endpoint, False) is None:
             endpoints.remove(endpoint)
         elif discover:
-            endpoints = recursive_discovering(endpoints, endpoint)
+            endpoints = await recursive_discovering(endpoints, endpoint)
     return endpoints
 
 
-def recursive_discovering(endpoints, endpoint):
+async def recursive_discovering(endpoints, endpoint):
     """
     Discover recursively new nodes.
     If new node found add it and try to found new node from his known nodes.
     """
-    news = parse_endpoints(get_request("network/peers")["peers"])
+    api = generate_duniterpy_endpoint_format(endpoint)
+    sub_client = Client(api)
+    news = await get_peers_among_leaves(sub_client)
+    await sub_client.close()
     for new in news:
-        if best_node(new, False) is not None and new not in endpoints:
+        if best_endpoint_address(new, False) is not None and new not in endpoints:
             endpoints.append(new)
-            recursive_discovering(endpoints, new)
+            await recursive_discovering(endpoints, new)
     return endpoints
 
 
+async def get_peers_among_leaves(client):
+    """
+    Browse among leaves of peers to retrieve the other peers’ endpoints
+    """
+    leaves = await client(network.peers, leaves=True)
+    peers = list()
+    for leaf in leaves["leaves"]:
+        await sleep(ASYNC_SLEEP + 0.05)
+        leaf_response = await client(network.peers, leaf=leaf)
+        peers.append(leaf_response["leaf"]["value"])
+    return parse_endpoints(peers)
+
+
 def parse_endpoints(rep):
     """
     endpoints[]{"domain", "ip4", "ip6", "pubkey"}
@@ -85,29 +104,52 @@ def parse_endpoints(rep):
     return endpoints
 
 
+def generate_duniterpy_endpoint_format(ep):
+    api = "BASIC_MERKLED_API " if ep["port"] != "443" else "BMAS "
+    api += ep.get("domain") + " " if "domain" in ep else ""
+    api += ep.get("ip4") + " " if "ip4" in ep else ""
+    api += ep.get("ip6") + " " if "ip6" in ep else ""
+    api += ep.get("port")
+    return api
+
+
+def singleton(class_):
+    instances = {}
+
+    def getinstance(*args, **kwargs):
+        if class_ not in instances:
+            instances[class_] = class_(*args, **kwargs)
+        return instances[class_]
+
+    return getinstance
+
+
+@singleton
 class EndPoint(object):
-    __instance = None
-
-    # Try to inheritate this part for all singleton classes
-    def __new__(cls):
-        if EndPoint.__instance is None:
-            EndPoint.__instance = object.__new__(cls)
-        return EndPoint.__instance
-
-    @click.pass_context
+    @pass_context
     def __init__(ctx, self):
         ep = dict()
-        if ctx.obj["PEER"]:
-            ep["domain"], ep["port"] = ctx.obj["PEER"].rsplit(":", 1)
+        peer = ctx.obj["PEER"]
+        if peer:
+            if ":" in peer:
+                ep["domain"], ep["port"] = peer.rsplit(":", 1)
+            else:
+                ep["domain"], ep["port"] = peer, "443"
         else:
             ep["domain"], ep["port"] = (
-                G1_TEST_DEFAULT_ENDPOINT
-                if ctx.obj["GTEST"]
-                else G1_DEFAULT_ENDPOINT
+                G1_TEST_DEFAULT_ENDPOINT if ctx.obj["GTEST"] else G1_DEFAULT_ENDPOINT
             )
         if ep["domain"].startswith("[") and ep["domain"].endswith("]"):
             ep["domain"] = ep["domain"][1:-1]
         self.ep = ep
+        api = "BMAS" if ep["port"] == "443" else "BASIC_MERKLED_API"
+        self.BMA_ENDPOINT = " ".join([api, ep["domain"], ep["port"]])
+
+
+@singleton
+class ClientInstance(object):
+    def __init__(self):
+        self.client = Client(EndPoint().BMA_ENDPOINT)
 
 
 def parse_endpoint(rep):
@@ -154,39 +196,7 @@ def check_ip(address):
         return 0
 
 
-def get_request(path, ep=None):
-    ep = EndPoint().ep
-    address = best_node(ep, False)
-    if address is None:
-        return address
-    url = "http://" + ep[address] + ":" + ep["port"] + "/" + path
-    if ep["port"] == "443":
-        url = "https://" + ep[address] + "/" + path
-    request = urllib.request.Request(url)
-    response = urllib.request.urlopen(request, timeout=CONNECTION_TIMEOUT)
-    encoding = response.info().get_content_charset("utf8")
-    return loads(response.read().decode(encoding))
-
-
-def post_request(path, postdata, ep=None):
-    ep = EndPoint().ep
-    address = best_node(ep, False)
-    if address is None:
-        return address
-    url = "http://" + ep[address] + ":" + ep["port"] + "/" + path
-    if ep["port"] == "443":
-        url = "https://" + ep[address] + "/" + path
-    request = urllib.request.Request(url, bytes(postdata, "utf-8"))
-    try:
-        response = urllib.request.urlopen(request, timeout=CONNECTION_TIMEOUT)
-    except urllib.error.URLError as e:
-        print(e, file=stderr)
-        exit(1)
-    encoding = response.info().get_content_charset("utf8")
-    return loads(response.read().decode(encoding))
-
-
-def best_node(ep, main):
+def best_endpoint_address(ep, main):
     s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
     s.settimeout(CONNECTION_TIMEOUT)
     addresses, port = ("domain", "ip6", "ip4"), int(ep["port"])
@@ -227,4 +237,8 @@ class HeadBlock(object):
         return HeadBlock.__instance
 
     def __init__(self):
-        self.head_block = get_request("blockchain/current")
+        self.head_block = self.get_head()
+
+    async def get_head(self):
+        client = ClientInstance().client
+        return await client(blockchain.current)
