@@ -18,41 +18,62 @@ along with Silkaj. If not, see <https://www.gnu.org/licenses/>.
 from re import compile, search
 import math
 from time import sleep
-import urllib
+from click import command, option, FloatRange
 
 from tabulate import tabulate
-from silkaj.network_tools import get_request, post_request, HeadBlock
-from silkaj.crypto_tools import (
-    get_publickey_from_seed,
-    sign_document_from_seed,
-    check_public_key,
-)
-from silkaj.tools import message_exit, CurrencySymbol
+from silkaj.network_tools import ClientInstance, HeadBlock
+from silkaj.crypto_tools import check_public_key
+from silkaj.tools import message_exit, CurrencySymbol, coroutine
 from silkaj.auth import auth_method
-from silkaj.wot import get_uid_from_pubkey
-from silkaj.money import get_sources, get_amount_from_pubkey, UDValue
-from silkaj.constants import NO_MATCHING_ID
+from silkaj.wot import is_member
+from silkaj.money import (
+    get_sources,
+    get_amount_from_pubkey,
+    UDValue,
+    amount_in_current_base,
+)
+from silkaj.constants import SOURCES_PER_TX
+
+from duniterpy.api.bma.tx import process
+from duniterpy.documents import BlockUID, Transaction
+from duniterpy.documents.transaction import OutputSource, Unlock, SIGParameter
 
 
-def send_transaction(
-    amount, amountUD, allSources, output, comment, outputBackChange, yes
+@command("tx", help="Send transaction")
+@option("--amount", type=FloatRange(0.01), help="Quantitative value")
+@option("--amountUD", type=float, help="Relative value")
+@option("--allSources", is_flag=True, help="Send all sources")
+@option(
+    "--output",
+    required=True,
+    help="Pubkey(s)’ recipients + optional checksum: <pubkey>[!checksum]:[<pubkey>[!checksum]]",
+)
+@option("--comment", default="", help="Comment")
+@option(
+    "--outputBackChange",
+    help="Pubkey recipient to send the rest of the transaction: <pubkey[!checksum]>",
+)
+@option("--yes", "-y", is_flag=True, help="Assume yes. Do not prompt confirmation")
+@coroutine
+async def send_transaction(
+    amount, amountud, allsources, output, comment, outputbackchange, yes
 ):
     """
     Main function
     """
-    tx_amount, comment = cmd_transaction(
-        amount, amountUD, allSources, output, comment, outputBackChange
-    )
-    seed = auth_method()
-    issuer_pubkey = get_publickey_from_seed(seed)
+    tx_amount = await transaction_amount(amount, amountud, allsources)
+    key = auth_method()
+    issuer_pubkey = key.pubkey
 
-    pubkey_amount = get_amount_from_pubkey(issuer_pubkey)[0]
+    pubkey_amount = await get_amount_from_pubkey(issuer_pubkey)
+    if allsources:
+        tx_amount = pubkey_amount[0]
     outputAddresses = output.split(":")
     check_transaction_values(
         comment,
         outputAddresses,
-        outputBackChange,
-        pubkey_amount < tx_amount * len(outputAddresses),
+        outputbackchange,
+        pubkey_amount[0] < tx_amount * len(outputAddresses),
         issuer_pubkey,
     )
 
@@ -60,8 +81,13 @@ def send_transaction(
         yes
         or input(
             tabulate(
-                transaction_confirmation(
-                    issuer_pubkey, pubkey_amount, tx_amount, outputAddresses, comment
+                await transaction_confirmation(
+                    issuer_pubkey,
+                    pubkey_amount[0],
+                    tx_amount,
+                    outputAddresses,
+                    outputbackchange,
+                    comment,
                 ),
                 tablefmt="fancy_grid",
             )
@@ -69,33 +95,22 @@ def send_transaction(
         )
         == "yes"
     ):
-        generate_and_send_transaction(
-            seed,
-            issuer_pubkey,
-            tx_amount,
-            outputAddresses,
-            comment,
-            allSources,
-            outputBackChange,
+        await handle_intermediaries_transactions(
+            key, issuer_pubkey, tx_amount, outputAddresses, comment, outputbackchange
         )
 
 
-def cmd_transaction(amount, amountUD, allSources, output, comment, outputBackChange):
+async def transaction_amount(amount, amountUD, allSources):
     """
-    Check command line interface arguments
+    Check command line interface amount option
+    Return transaction amount
     """
     if not (amount or amountUD or allSources):
         message_exit("--amount nor --amountUD nor --allSources is set")
-    if not output:
-        message_exit("--output is not set")
     if amount:
-        tx_amount = float(amount) * 100
+        return round(amount * 100)
     if amountUD:
-        tx_amount = float(amountUD) * UDValue().ud_value
-    if allSources:
-        tx_amount = 0
-    comment = comment if comment else ""
-    return tx_amount, comment
+        return round(amountUD * await UDValue().ud_value)
 
 
 def check_transaction_values(
@@ -115,17 +130,17 @@ def check_transaction_values(
         )
 
 
-def transaction_confirmation(
-    issuer_pubkey, pubkey_amount, tx_amount, outputAddresses, comment
+async def transaction_confirmation(
+    issuer_pubkey, pubkey_amount, tx_amount, outputAddresses, outputBackChange, comment
 ):
     """
     Generate transaction confirmation
     """
 
-    currency_symbol = CurrencySymbol().symbol
+    currency_symbol = await CurrencySymbol().symbol
     tx = list()
     tx.append(
-        ["pubkey’s amount before tx", str(pubkey_amount / 100) + " " + currency_symbol]
+        ["pubkey’s balance before tx", str(pubkey_amount / 100) + " " + currency_symbol]
     )
     tx.append(
         [
@@ -136,79 +151,84 @@ def transaction_confirmation(
     tx.append(
         [
             "tx amount (relative)",
-            str(round(tx_amount / UDValue().ud_value, 4)) + " UD " + currency_symbol,
+            str(round(tx_amount / await UDValue().ud_value, 4))
+            + " UD "
+            + currency_symbol,
         ]
     )
     tx.append(
         [
-            "pubkey’s amount after tx",
+            "pubkey’s balance after tx",
             str(((pubkey_amount - tx_amount * len(outputAddresses)) / 100))
             + " "
             + currency_symbol,
         ]
     )
     tx.append(["from (pubkey)", issuer_pubkey])
-    id_from = get_uid_from_pubkey(issuer_pubkey)
-    if id_from is not NO_MATCHING_ID:
-        tx.append(["from (id)", id_from])
+    id_from = await is_member(issuer_pubkey)
+    if id_from:
+        tx.append(["from (id)", id_from["uid"]])
     for outputAddress in outputAddresses:
         tx.append(["to (pubkey)", outputAddress])
-        id_to = get_uid_from_pubkey(outputAddress)
-        if id_to is not NO_MATCHING_ID:
-            tx.append(["to (id)", id_to])
+        id_to = await is_member(outputAddress)
+        if id_to:
+            tx.append(["to (id)", id_to["uid"]])
+    if outputBackChange:
+        tx.append(["Backchange (pubkey)", outputBackChange])
+        id_backchange = await is_member(outputBackChange)
+        if id_backchange:
+            tx.append(["Backchange (id)", id_backchange["uid"]])
     tx.append(["comment", comment])
     return tx
 
 
-def generate_and_send_transaction(
-    seed,
-    issuers,
-    AmountTransfered,
-    outputAddresses,
-    Comment="",
-    all_input=False,
-    OutputbackChange=None,
-):
+async def get_list_input_for_transaction(pubkey, TXamount):
+    listinput, amount = await get_sources(pubkey)
 
+    # generate final list source
+    listinputfinal = []
+    totalAmountInput = 0
+    intermediatetransaction = False
+    for input in listinput:
+        listinputfinal.append(input)
+        totalAmountInput += amount_in_current_base(input)
+        TXamount -= amount_in_current_base(input)
+        # if more than 40 sources, it's an intermediate transaction
+        if len(listinputfinal) >= SOURCES_PER_TX:
+            intermediatetransaction = True
+            break
+        if TXamount <= 0:
+            break
+    if TXamount > 0 and not intermediatetransaction:
+        message_exit("Error: you don't have enough money")
+    return listinputfinal, totalAmountInput, intermediatetransaction
+
+
+async def handle_intermediaries_transactions(
+    key, issuers, AmountTransfered, outputAddresses, Comment="", OutputbackChange=None
+):
+    client = ClientInstance().client
     while True:
-        listinput_and_amount = get_list_input_for_transaction(
-            issuers, AmountTransfered * len(outputAddresses), all_input
+        listinput_and_amount = await get_list_input_for_transaction(
+            issuers, AmountTransfered * len(outputAddresses)
         )
         intermediatetransaction = listinput_and_amount[2]
 
         if intermediatetransaction:
             totalAmountInput = listinput_and_amount[1]
-            print("Generate Change Transaction")
-            print("   - From:    " + issuers)
-            print("   - To:      " + issuers)
-            print("   - Amount:  " + str(totalAmountInput / 100))
-            transaction = generate_transaction_document(
+            await generate_and_send_transaction(
+                key,
                 issuers,
                 totalAmountInput,
                 listinput_and_amount,
                 issuers,
                 "Change operation",
             )
-            transaction += sign_document_from_seed(transaction, seed) + "\n"
-            post_request(
-                "tx/process", "transaction=" + urllib.parse.quote_plus(transaction)
-            )
-            print("Change Transaction successfully sent.")
             sleep(1)  # wait 1 second before sending a new transaction
 
         else:
-            print("Generate Transaction:")
-            print("   - From:    " + issuers)
-            for outputAddress in outputAddresses:
-                print("   - To:      " + outputAddress)
-            if all_input:
-                print("   - Amount:  " + str(listinput_and_amount[1] / 100))
-            else:
-                print(
-                    "   - Amount:  "
-                    + str(AmountTransfered / 100 * len(outputAddresses))
-                )
-            transaction = generate_transaction_document(
+            await generate_and_send_transaction(
+                key,
                 issuers,
                 AmountTransfered,
                 listinput_and_amount,
@@ -216,16 +236,61 @@ def generate_and_send_transaction(
                 Comment,
                 OutputbackChange,
             )
-            transaction += sign_document_from_seed(transaction, seed) + "\n"
-
-            post_request(
-                "tx/process", "transaction=" + urllib.parse.quote_plus(transaction)
-            )
-            print("Transaction successfully sent.")
+            await client.close()
             break
 
 
-def generate_transaction_document(
+async def generate_and_send_transaction(
+    key,
+    issuers,
+    amount,
+    listinput_and_amount,
+    outputAddresses,
+    Comment,
+    OutputbackChange=None,
+):
+    """
+    Display sent transaction
+    Generate, sign, and send transaction document
+    """
+    intermediate_tx = listinput_and_amount[2]
+    if intermediate_tx:
+        print("Generate Change Transaction")
+    else:
+        print("Generate Transaction:")
+    print("   - From:    " + issuers)
+    if isinstance(outputAddresses, str):
+        display_sent_tx(outputAddresses, amount)
+    else:
+        for outputAddress in outputAddresses:
+            display_sent_tx(outputAddress, amount)
+        if len(outputAddresses) > 1:
+            print("   - Total:   " + str(amount / 100 * len(outputAddresses)))
+
+    client = ClientInstance().client
+    transaction = await generate_transaction_document(
+        issuers,
+        amount,
+        listinput_and_amount,
+        outputAddresses,
+        Comment,
+        OutputbackChange,
+    )
+    transaction.sign([key])
+    response = await client(process, transaction.signed_raw())
+    if response.status == 200:
+        print("Transaction successfully sent.")
+    else:
+        message_exit(
+            "Error while publishing transaction: {0}".format(await response.text())
+        )
+
+
+def display_sent_tx(outputAddress, amount):
+    print("   - To:     ", outputAddress, "\n   - Amount: ", amount / 100)
+
+
+async def generate_transaction_document(
     issuers,
     AmountTransfered,
     listinput_and_amount,
@@ -239,9 +304,9 @@ def generate_transaction_document(
     listinput = listinput_and_amount[0]
     totalAmountInput = listinput_and_amount[1]
 
-    head_block = HeadBlock().head_block
+    head_block = await HeadBlock().head_block
     currency_name = head_block["currency"]
-    blockstamp_current = str(head_block["number"]) + "-" + str(head_block["hash"])
+    blockstamp_current = BlockUID(head_block["number"], head_block["hash"])
     curentUnitBase = head_block["unitbase"]
 
     if not OutputbackChange:
@@ -267,28 +332,31 @@ def generate_transaction_document(
     rest = totalAmountInput - totalAmountTransfered
     generate_output(listoutput, curentUnitBase, rest, OutputbackChange)
 
+    # Unlocks
+    unlocks = generate_unlocks(listinput)
+
     # Generate transaction document
     ##############################
 
-    transaction_document = "Version: 10\n"
-    transaction_document += "Type: Transaction\n"
-    transaction_document += "Currency: " + currency_name + "\n"
-    transaction_document += "Blockstamp: " + blockstamp_current + "\n"
-    transaction_document += "Locktime: 0\n"
-    transaction_document += "Issuers:\n"
-    transaction_document += issuers + "\n"
-    transaction_document += "Inputs:\n"
-    for input in listinput:
-        transaction_document += input + "\n"
-    transaction_document += "Unlocks:\n"
-    for i in range(0, len(listinput)):
-        transaction_document += str(i) + ":SIG(0)\n"
-    transaction_document += "Outputs:\n"
-    for output in listoutput:
-        transaction_document += output + "\n"
-    transaction_document += "Comment: " + Comment + "\n"
+    return Transaction(
+        version=10,
+        currency=currency_name,
+        blockstamp=blockstamp_current,
+        locktime=0,
+        issuers=[issuers],
+        inputs=listinput,
+        unlocks=unlocks,
+        outputs=listoutput,
+        comment=Comment,
+        signatures=[],
+    )
 
-    return transaction_document
+
+def generate_unlocks(listinput):
+    unlocks = list()
+    for i in range(0, len(listinput)):
+        unlocks.append(Unlock(index=i, parameters=[SIGParameter(0)]))
+    return unlocks
 
 
 def generate_output(listoutput, unitbase, rest, recipient_address):
@@ -298,39 +366,15 @@ def generate_output(listoutput, unitbase, rest, recipient_address):
         if outputAmount > 0:
             outputAmount = int(outputAmount / math.pow(10, unitbase))
             listoutput.append(
-                str(outputAmount)
-                + ":"
-                + str(unitbase)
-                + ":SIG("
-                + recipient_address
-                + ")"
+                OutputSource(
+                    amount=str(outputAmount),
+                    base=unitbase,
+                    condition="SIG({0})".format(recipient_address),
+                )
             )
         unitbase = unitbase - 1
 
 
-def get_list_input_for_transaction(pubkey, TXamount, allinput=False):
-    listinput, amount = get_sources(pubkey)
-
-    # generate final list source
-    listinputfinal = []
-    totalAmountInput = 0
-    intermediatetransaction = False
-    for input in listinput:
-        listinputfinal.append(input)
-        inputsplit = input.split(":")
-        totalAmountInput += int(inputsplit[0]) * 10 ** int(inputsplit[1])
-        TXamount -= int(inputsplit[0]) * 10 ** int(inputsplit[1])
-        # if more 40 sources, it's an intermediate transaction
-        if len(listinputfinal) >= 40:
-            intermediatetransaction = True
-            break
-        if TXamount <= 0 and not allinput:
-            break
-    if TXamount > 0 and not intermediatetransaction:
-        message_exit("Error: you don't have enough money")
-    return listinputfinal, totalAmountInput, intermediatetransaction
-
-
 def checkComment(Comment):
     if len(Comment) > 255:
         message_exit("Error: Comment is too long")
