@@ -15,15 +15,19 @@ You should have received a copy of the GNU Affero General Public License
 along with Silkaj. If not, see <https://www.gnu.org/licenses/>.
 """
 
+from click import command, argument
 from time import time
 from tabulate import tabulate
 from collections import OrderedDict
+from asyncio import sleep
+from duniterpy.api.bma import wot, blockchain
+from duniterpy.api.errors import DuniterError
 
-from silkaj.network_tools import get_request
+from silkaj.network_tools import ClientInstance
 from silkaj.crypto_tools import check_public_key
-from silkaj.tools import message_exit, convert_time
+from silkaj.tools import message_exit, convert_time, coroutine
 from silkaj.blockchain_tools import BlockchainParams
-from silkaj.constants import NO_MATCHING_ID
+from silkaj.constants import ASYNC_SLEEP
 
 
 def get_sent_certifications(certs, time_first_block, params):
@@ -40,20 +44,29 @@ def get_sent_certifications(certs, time_first_block, params):
     return sent, expire
 
 
-def received_sent_certifications(id):
+@command(
+    "wot",
+    help="Check received and sent certifications and consult the membership status of any given identity",
+)
+@argument("id")
+@coroutine
+async def received_sent_certifications(id):
     """
     get searched id
     get id of received and sent certifications
-    display on a chart the result with the numbers
+    display in a table the result with the numbers
     """
-    time_first_block = get_request("blockchain/block/1")["time"]
-    id_certs = get_informations_for_identity(id)
+    client = ClientInstance().client
+    first_block = await client(blockchain.block, 1)
+    time_first_block = first_block["time"]
+    id_certs = await get_informations_for_identity(id)
     certifications = OrderedDict()
-    params = BlockchainParams().params
+    params = await BlockchainParams().params
     for certs in id_certs["uids"]:
         if certs["uid"].lower() == id.lower():
             pubkey = id_certs["pubkey"]
-            req = get_request("wot/requirements/" + pubkey)["identities"][0]
+            req = await client(wot.requirements, pubkey)
+            req = req["identities"][0]
             certifications["received_expire"] = list()
             certifications["received"] = list()
             for cert in certs["others"]:
@@ -72,7 +85,7 @@ def received_sent_certifications(id):
                 len(certifications["sent"]) if "sent" in certifications else 0
             )
             print(
-                "{0} ({1}) from block #{2}\nreceived {3} and sent {4}/{5} certifications:\n{6}\n".format(
+                "{0} ({1}) from block #{2}\nreceived {3} and sent {4}/{5} certifications:\n{6}\n{7}\n".format(
                     id,
                     pubkey[:5] + "…",
                     certs["meta"]["timestamp"][:15] + "…",
@@ -85,9 +98,11 @@ def received_sent_certifications(id):
                         tablefmt="orgtbl",
                         stralign="center",
                     ),
+                    "✔: Certifications written into the blockchain",
                 )
             )
-            membership_status(certifications, certs, pubkey, req)
+            await membership_status(certifications, certs, pubkey, req)
+    await client.close()
 
 
 def cert_written_in_the_blockchain(written_certs, certifieur):
@@ -97,16 +112,18 @@ def cert_written_in_the_blockchain(written_certs, certifieur):
     return certifieur["uids"][0]
 
 
-def membership_status(certifications, certs, pubkey, req):
-    params = BlockchainParams().params
+async def membership_status(certifications, certs, pubkey, req):
+    params = await BlockchainParams().params
     if len(certifications["received"]) >= params["sigQty"]:
         print(
-            "Membership expiration due to certifications expirations: "
+            "Membership expiration due to certification expirations: "
             + certifications["received_expire"][
                 len(certifications["received"]) - params["sigQty"]
             ]
         )
-    member = is_member(pubkey, certs["uid"])
+    member = await is_member(pubkey)
+    if member:
+        member = True
     print("member:", member)
     if req["revoked"]:
         print(
@@ -138,80 +155,103 @@ def date_approximation(block_id, time_first_block, avgentime):
     return time_first_block + block_id * avgentime
 
 
-def id_pubkey_correspondence(id_pubkey):
+@command("id", help="Find corresponding identity or pubkey from pubkey or identity")
+@argument("id_pubkey")
+@coroutine
+async def id_pubkey_correspondence(id_pubkey):
+    client = ClientInstance().client
     if check_public_key(id_pubkey, False):
-        print(
-            "{} public key corresponds to identity: {}".format(
-                id_pubkey, get_uid_from_pubkey(id_pubkey)
+        try:
+            idty = await identity_of(id_pubkey)
+            print(
+                "{} public key corresponds to identity: {}".format(
+                    id_pubkey, idty["uid"]
+                )
             )
-        )
+        except:
+            message_exit("No matching identity")
     else:
-        pubkeys = get_informations_for_identities(id_pubkey)
-        if pubkeys == NO_MATCHING_ID:
-            print(NO_MATCHING_ID)
-        else:
-            print("Public keys found matching '{}':\n".format(id_pubkey))
-            for pubkey in pubkeys:
-                print("→", pubkey["pubkey"], end=" ")
-                try:
-                    print(
-                        "↔ " + get_request("wot/identity-of/" + pubkey["pubkey"])["uid"]
-                    )
-                except:
-                    print("")
+        pubkeys = await wot_lookup(id_pubkey)
+        print("Public keys found matching '{}':\n".format(id_pubkey))
+        for pubkey in pubkeys:
+            print("→", pubkey["pubkey"], end=" ")
+            try:
+                corresponding_id = await client(wot.identity_of, pubkey["pubkey"])
+                print("↔ " + corresponding_id["uid"])
+            except:
+                print("")
+    await client.close()
 
 
-def get_informations_for_identity(id):
+async def get_informations_for_identity(id):
     """
     Check that the id is present on the network
     many identities could match
     return the one searched
     """
-    certs_req = get_informations_for_identities(id)
-    if certs_req == NO_MATCHING_ID:
-        message_exit(NO_MATCHING_ID)
+    certs_req = await wot_lookup(id)
     for certs_id in certs_req:
         if certs_id["uids"][0]["uid"].lower() == id.lower():
             return certs_id
-    message_exit(NO_MATCHING_ID)
+    message_exit("No matching identity")
 
 
-def get_uid_from_pubkey(pubkey):
+async def identity_of(pubkey_uid):
+    """
+    Only works for members
+    Not able to get corresponding uid from a non-member identity
+    Able to know if an identity is member or not
+    """
+    client = ClientInstance().client
     try:
-        results = get_request("wot/lookup/" + pubkey)
+        return await client(wot.identity_of, pubkey_uid)
+    except DuniterError as e:
+        raise DuniterError(e)
+    except ValueError as e:
+        pass
+
+
+async def is_member(pubkey_uid):
+    """
+    Check identity is member
+    If member, return corresponding identity, else: False
+    """
+    try:
+        return await identity_of(pubkey_uid)
     except:
-        return NO_MATCHING_ID
-    i, results = 0, results["results"]
-    while i < len(results):
-        if results[i]["uids"][0]["uid"] != pubkey:
-            return results[i]["uids"][0]["uid"]
-        i += 1
+        return False
 
 
-def get_informations_for_identities(identifier):
+async def wot_lookup(identifier):
     """
     :identifier: identity or pubkey in part or whole
     Return received and sent certifications lists of matching identities
     if one identity found
     """
+    client = ClientInstance().client
     try:
-        results = get_request("wot/lookup/" + identifier)
-    except:
-        return NO_MATCHING_ID
-    return results["results"]
+        results = await client(wot.lookup, identifier)
+        return results["results"]
+    except DuniterError as e:
+        message_exit(e.message)
+    except ValueError as e:
+        pass
 
 
-def is_member(pubkey, uid):
-    members = get_request("wot/members")["results"]
-    for member in members:
-        if pubkey in member["pubkey"] and uid in member["uid"]:
-            return True
-    return False
+async def identities_from_pubkeys(pubkeys, uids):
+    """
+    Make list of pubkeys unique, and remove empty strings
+    Request identities
+    """
+    if not uids:
+        return list()
 
-
-def get_pubkey_from_id(uid):
-    members = get_request("wot/members")["results"]
-    for member in members:
-        if uid == member["uid"]:
-            return member["pubkey"]
-    return NO_MATCHING_ID
+    uniq_pubkeys = list(filter(None, set(pubkeys)))
+    identities = list()
+    for pubkey in uniq_pubkeys:
+        try:
+            identities.append(await identity_of(pubkey))
+        except Exception as e:
+            pass
+        await sleep(ASYNC_SLEEP)
+    return identities
